@@ -79,7 +79,12 @@ impl TargetMatcher for targ::TargetInfo {
             },
             Family(fam) => self.families.contains(fam),
             HasAtomic(has_atomic) => self.has_atomics.contains(*has_atomic),
-            Os(os) => Some(os) == self.os.as_ref(),
+            Os(os) => match &self.os {
+                Some(self_os) => os == self_os,
+                // os = "none" means it should be matched against None. Note that this is different
+                // from "env" above.
+                None => os.as_str() == "none",
+            },
             PointerWidth(w) => *w == self.pointer_width,
             Vendor(ven) => match &self.vendor {
                 Some(v) => ven == v,
@@ -116,6 +121,23 @@ impl TargetMatcher for target_lexicon::Triple {
                 } else if arch == &targ::Arch::bpf {
                     self.architecture == Architecture::Bpfeb
                         || self.architecture == Architecture::Bpfel
+                } else if arch == &targ::Arch::x86_64 {
+                    self.architecture == Architecture::X86_64
+                        || self.architecture == Architecture::X86_64h
+                } else if arch == &targ::Arch::mips32r6 {
+                    matches!(
+                        self.architecture,
+                        Architecture::Mips32(
+                            Mips32Architecture::Mipsisa32r6 | Mips32Architecture::Mipsisa32r6el
+                        )
+                    )
+                } else if arch == &targ::Arch::mips64r6 {
+                    matches!(
+                        self.architecture,
+                        Architecture::Mips64(
+                            Mips64Architecture::Mipsisa64r6 | Mips64Architecture::Mipsisa64r6el
+                        )
+                    )
                 } else {
                     match arch.0.parse::<Architecture>() {
                         Ok(a) => match (self.architecture, a) {
@@ -148,13 +170,13 @@ impl TargetMatcher for target_lexicon::Triple {
                     OperatingSystem::VxWorks => env == &targ::Env::gnu,
                     OperatingSystem::Freebsd => match self.architecture {
                         Architecture::Arm(ArmArchitecture::Armv6 | ArmArchitecture::Armv7) => {
-                            env == &targ::Env::gnueabihf
+                            env == &targ::Env::gnu
                         }
                         _ => env.0.is_empty(),
                     },
                     OperatingSystem::Netbsd => match self.architecture {
                         Architecture::Arm(ArmArchitecture::Armv6 | ArmArchitecture::Armv7) => {
-                            env == &targ::Env::eabihf
+                            env.0.is_empty()
                         }
                         _ => env.0.is_empty(),
                     },
@@ -165,6 +187,9 @@ impl TargetMatcher for target_lexicon::Triple {
                         Environment::LinuxKernel => env == &targ::Env::gnu,
                         _ => env.0.is_empty(),
                     },
+                    OperatingSystem::WasiP1 => env == &targ::Env::p1,
+                    OperatingSystem::WasiP2 => env == &targ::Env::p2,
+                    OperatingSystem::Wasi => env.0.is_empty() || env == &targ::Env::p1,
                     _ => {
                         if env.0.is_empty() {
                             matches!(
@@ -176,6 +201,7 @@ impl TargetMatcher for target_lexicon::Triple {
                                     | Environment::Eabi
                                     | Environment::Eabihf
                                     | Environment::Sim
+                                    | Environment::None
                             )
                         } else {
                             match env.0.parse::<Environment>() {
@@ -219,6 +245,11 @@ impl TargetMatcher for target_lexicon::Triple {
                                                 | Environment::Uclibceabi
                                                 | Environment::Uclibceabihf
                                         )
+                                    } else if env == &targ::Env::newlib {
+                                        matches!(
+                                            self.operating_system,
+                                            OperatingSystem::Horizon | OperatingSystem::Espidf
+                                        )
                                     } else {
                                         self.environment == e
                                     }
@@ -230,11 +261,11 @@ impl TargetMatcher for target_lexicon::Triple {
                 }
             }
             Family(fam) => {
-                use target_lexicon::OperatingSystem::{
+                use OperatingSystem::{
                     Aix, AmdHsa, Bitrig, Cloudabi, Cuda, Darwin, Dragonfly, Emscripten, Espidf,
-                    Freebsd, Fuchsia, Haiku, Hermit, Horizon, Illumos, Ios, L4re, Linux, MacOSX,
-                    Nebulet, Netbsd, None_, Openbsd, Redox, Solaris, Tvos, Uefi, Unknown, VxWorks,
-                    Wasi, Watchos, Windows,
+                    Freebsd, Fuchsia, Haiku, Hermit, Horizon, Hurd, Illumos, Ios, L4re, Linux,
+                    MacOSX, Nebulet, Netbsd, None_, Openbsd, Redox, Solaris, Tvos, Uefi, Unknown,
+                    Visionos, VxWorks, Wasi, WasiP1, WasiP2, Watchos, Windows,
                 };
                 match self.operating_system {
                     AmdHsa | Bitrig | Cloudabi | Cuda | Hermit | Nebulet | None_ | Uefi => false,
@@ -245,6 +276,7 @@ impl TargetMatcher for target_lexicon::Triple {
                     | Freebsd
                     | Fuchsia
                     | Haiku
+                    | Hurd
                     | Illumos
                     | Ios
                     | L4re
@@ -255,6 +287,7 @@ impl TargetMatcher for target_lexicon::Triple {
                     | Redox
                     | Solaris
                     | Tvos
+                    | Visionos
                     | VxWorks
                     | Watchos => fam == &crate::targets::Family::unix,
                     Emscripten => {
@@ -284,7 +317,7 @@ impl TargetMatcher for target_lexicon::Triple {
                             false
                         }
                     }
-                    Wasi => fam == &crate::targets::Family::wasm,
+                    Wasi | WasiP1 | WasiP2 => fam == &crate::targets::Family::wasm,
                     Windows => fam == &crate::targets::Family::windows,
                     // I really dislike non-exhaustive :(
                     _ => false,
@@ -295,31 +328,53 @@ impl TargetMatcher for target_lexicon::Triple {
                 // this.
                 false
             }
-            Os(os) => match os.0.parse::<OperatingSystem>() {
-                Ok(o) => match self.environment {
-                    Environment::HermitKernel => os == &targ::Os::hermit,
-                    _ => self.operating_system == o,
-                },
-                Err(_) => {
-                    // Handle special case for darwin/macos, where the triple is
-                    // "darwin", but rustc identifies the OS as "macos"
-                    if os == &targ::Os::macos && self.operating_system == OperatingSystem::Darwin {
-                        true
-                    } else {
-                        // For android, the os is still linux, but the environment is android
-                        os == &targ::Os::android
-                            && self.operating_system == OperatingSystem::Linux
-                            && (self.environment == Environment::Android
-                                || self.environment == Environment::Androideabi)
+            Os(os) => {
+                if os == &targ::Os::wasi
+                    && matches!(
+                        self.operating_system,
+                        OperatingSystem::WasiP1 | OperatingSystem::WasiP2
+                    )
+                {
+                    return true;
+                }
+                match os.0.parse::<OperatingSystem>() {
+                    Ok(o) => match self.environment {
+                        Environment::HermitKernel => os == &targ::Os::hermit,
+                        _ => self.operating_system == o,
+                    },
+                    Err(_) => {
+                        // Handle special case for darwin/macos, where the triple is
+                        // "darwin", but rustc identifies the OS as "macos"
+                        if os == &targ::Os::macos
+                            && self.operating_system == OperatingSystem::Darwin
+                        {
+                            true
+                        } else {
+                            // For android, the os is still linux, but the environment is android
+                            os == &targ::Os::android
+                                && self.operating_system == OperatingSystem::Linux
+                                && (self.environment == Environment::Android
+                                    || self.environment == Environment::Androideabi)
+                        }
                     }
                 }
-            },
+            }
             Panic(_) => {
                 // panic support depends on the OS. Assume false for this.
                 false
             }
             Vendor(ven) => match ven.0.parse::<target_lexicon::Vendor>() {
-                Ok(v) => self.vendor == v,
+                Ok(v) => {
+                    if self.vendor == v {
+                        true
+                    } else if let target_lexicon::Vendor::Custom(custom) = &self.vendor {
+                        matches!(custom.as_str(), "esp" | "esp32" | "esp32s2" | "esp32s3")
+                            && (v == target_lexicon::Vendor::Espressif
+                                || v == target_lexicon::Vendor::Unknown)
+                    } else {
+                        false
+                    }
+                }
                 Err(_) => false,
             },
             PointerWidth(pw) => {
@@ -400,7 +455,7 @@ pub enum Predicate<'a> {
     /// when compiling without optimizations.
     DebugAssertions,
     /// [Enabled](https://doc.rust-lang.org/reference/conditional-compilation.html#proc_macro) for
-    /// crates of the proc_macro type.
+    /// crates of the `proc_macro` type.
     ProcMacro,
     /// A [`feature = "<name>"`](https://doc.rust-lang.org/nightly/cargo/reference/features.html)
     Feature(&'a str),
@@ -505,7 +560,7 @@ impl Expression {
 
     /// Evaluates the expression, using the provided closure to determine the value of
     /// each predicate, which are then combined into a final result depending on the
-    /// functions not(), all(), or any() in the expression.
+    /// functions `not()`, `all()`, or `any()` in the expression.
     ///
     /// `eval_predicate` typically returns `bool`, but may return any type that implements
     /// the `Logic` trait.
@@ -562,7 +617,7 @@ impl Expression {
     {
         let mut result_stack = SmallVec::<[T; 8]>::new();
 
-        // We store the expression as postfix, so just evaluate each license
+        // We store the expression as postfix, so just evaluate each component
         // requirement in the order it comes, and then combining the previous
         // results according to each operator as it comes
         for node in self.expr.iter() {
@@ -623,7 +678,7 @@ impl Expression {
     }
 }
 
-/// [`PartialEq`] will do a **syntactical** comparaison, so will just check if both
+/// [`PartialEq`] will do a **syntactical** comparison, so will just check if both
 /// expressions have been parsed from the same string, **not** if they are semantically
 /// equivalent.
 ///
